-- (c) David Cunningham 2009, Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php

function transition_entity(ent, transition)
        for i=0,ent.numSubEntities-1 do
                local mname = ent:getMaterialName(i)
                if mname:sub(-1) == "'" then
                        mname = mname:sub(1,#mname-1)
                end
                if transition == 0 then
                        --ent.visible = true
                        ent:setMaterial(i,mname)
                        --elseif transition == 1 then
                        --        ent.visible = false
                else
                        --ent.visible = true
                        if registered_material_get(mname,"stipple") == false and registered_material_get(mname,"alpha") == false then
                                ent:setMaterial(i,mname.."'")
                        end
                end
        end
end

local function get_rot (x)
        if x == nil then return Q_ID end
        if getmetatable(x) == getmetatable(V_ID) then 
                return Quat(x.z,V_UP) * Quat(x.y,V_NORTH) * Quat(x.x,V_EAST)
        end
        return x
end

BaseClass = {
        renderingDistance=400;
        init = function (persistent,class)
                local gfxMesh = persistent.gfxMesh or class.gfxMesh or persistent.className..".mesh"
                gfxMesh = fqn_ex(gfxMesh, class.name):sub(2)
                persistent:hintAdvancePrepare("Mesh",gfxMesh)
                if persistent.textures == nil then return end
                for _,v in ipairs(persistent.textures) do
                        v = fqn_ex(v, class.name):sub(2)
                        persistent:hintAdvancePrepare("Texture", v)
                end
        end;
        activate=function (persistent, instance, class, gfxRoot)
                --echo("Activating: "..persistent.name.." ("..persistent.className..")")
                instance.persistent = persistent
                instance.class = class
                instance.renderingDistance = class.renderingDistance
                instance.gfx = gfxRoot:createChild()
                local gfxMesh = persistent.gfxMesh or class.gfxMesh or persistent.className..".mesh"
                gfxMesh = fqn_ex(gfxMesh, class.name):sub(2)
                instance.entity = sm:createMovableObject(persistent.name,"Entity",{mesh=gfxMesh})
                if instance.entity.hasSkeleton then instance.entity:setAllBonesManuallyControlled(true) end
                instance.entity.renderingDistance = instance.renderingDistance
                instance.entity:setCustomParameterAll(0,1,0,0,0)
                instance.entity.castShadows = persistent.castShadows == true
                instance.gfx.position = persistent.spawnPos
                instance.gfx.orientation = get_rot(persistent.rot)
                instance.gfx:attachObject(instance.entity)
        end;
        setFade=function(persistent, instance, fade, transition)
                --print(persistent.name,fade,transition)
                local ent = instance.entity
                -- with stipple, just setting fade is enough
                if transition and ent then
                        -- slow path
                        transition_entity(ent, transition)
                end
                ent:setCustomParameterAll(0,0,fade)
                ent.visible = fade > 0.001
        end;
        deactivate=function(persistent, instance)
                --echo("Deactivating: "..persistent.name.." ("..persistent.className..")")
                local pos = persistent.spawnPos
                persistent:updateSphere(pos.x,pos.y,pos.z,instance.renderingDistance)
                instance.gfx = safe_destroy(instance.gfx)
                instance.entity = safe_destroy(instance.entity)
                return persistent.norespawn
        end;
        reload=function(persistent)
                local class = persistent.class
                local gfxMesh = persistent.gfxMesh or class.gfxMesh or persistent.className..".mesh"
                gfxMesh = fqn_ex(gfxMesh, class.name):sub(2)
                local mesh = get_mesh(gfxMesh)
                mesh:reload()
                if mesh.hasSkeleton then
                        mesh.skeleton:reload()
                end
        end;
        setPush=function() end;
        setPull=function() end;
        setShouldSteerLeft=function() end;
        setShouldSteerRight=function() end;
        setSpecialLeft=function() end;
        setSpecialRight=function() end;
        setSpecialUp=function() end;
        setSpecialDown=function() end;
        setAltLeft=function() end;
        setAltRight=function() end;
        setAltUp=function() end;
        setAltDown=function() end;
        special=function() end;
        setHandBrake=function() end;
        realign=function() end;
}

ColClass = extends (BaseClass) {
        activate=function (persistent,instance,class,gfxRoot,physicsWorld)
                BaseClass.activate(persistent,instance,class,gfxRoot,physicsWorld)
                local colMesh = class.colMesh or persistent.className..".tcol"
                colMesh = fqn_ex(colMesh, class.name):sub(2)
                --echo("adding: "..tostring(persistent).." with "..colMesh)
                instance.world = physicsWorld
                instance.body = physicsWorld:addRigidBody(
                        colMesh,
                        persistent.spawnPos,
                        get_rot(persistent.rot)
                )
                instance.body.owner = persistent;
                if persistent.floating then
                        instance.body:deactivate()
                end
                -- this causes an alloc so do them here where the code is cold
                instance.body.updateCallback = function (px,py,pz,rw,rx,ry,rz)
                        local node = instance.gfx
                        -- can be nil
                        if node ~= nil and alive(node) then
                                node:setPosition(px,py,pz)
                                node:setOrientation(rw,rx,ry,rz)
                        end
                        persistent:updateSphere(px,py,pz,instance.renderingDistance)
                end
        end;
        deactivate=function(persistent, instance)
                instance.body = safe_destroy(instance.body)
                return BaseClass.deactivate(persistent, instance)
        end;

        getSpeed = function (persistent)
                if not persistent.activated then error("Not activated: "..persistent.persistent.name) end
                local rb = persistent.instance.body
                return #rb.linearVelocity
        end;
        setSpeed = function (persistent,speed)
                if not persistent.activated then error("Not activated: "..persistent.persistent.name) end
                local rb = persistent.instance.body
                rb.linearVelocity = rb.linearVelocity.normalised * speed
                rb:activate();
        end;
        boost = function (persistent,speed)
                speed = speed or 1
                persistent:setSpeed(persistent:getSpeed()+speed)
        end;
        realign = function (persistent)
                if not persistent.activated then error("Not activated: "..persistent.name) end
                local rb = persistent.instance.body
                rb.worldOrientation = V_NORTH:getRotationTo(rb.worldOrientation * V_FORWARDS * Vector3(1,1,0));
                rb.worldPosition = rb.worldPosition + Vector3(0,0,3);
                rb.angularVelocity = V_ZERO
                rb.linearVelocity = V_ZERO
                rb:activate() 
        end;
        reload=function(persistent)
                BaseClass.reload(persistent)
                local class = persistent.class
                local colMesh = class.colMesh or persistent.className..".tcol"
                colMesh = fqn_ex(colMesh, class.name):sub(2)
                local mesh = pw:getMesh(colMesh)
                mesh:reload()
        end;
}

-- vim: shiftwidth=8:tabstop=8:expandtab
