-- (c) David Cunningham 2009, Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php

print("Loading anaglyph_camera.lua")

local function attach(self, new_parent)
        new_parent:addChild(self.root)
        new_parent:addChild(gfx.skyNode)
end

local function detach(self)
        if self.root.parentSceneNode then
                self.root.parentSceneNode:removeChild(self.root)
        end
        if gfx.skyNode.parentSceneNode then
                gfx.skyNode.parentSceneNode:removeChild(gfx.skyNode)
        end
end

MonoCamera = { }

function MonoCamera.new(scnmgr,parent,render_target,z,left,top,width,height)
        local self = { }

        self.scnmgr = scnmgr

        self.cam = scnmgr:createCamera("MonoCamera")
        self.cam.orientation = Quat(90,V_EAST)
        self.root = parent:createChild()
        self.root:attachObject(self.cam)
        
        self.viewport = render_target:addViewport(self.cam,z,left,top,width,height)

        return make_instance(self,MonoCamera,function (k)
                if k=="destroy" or k=="init" or k=="detach" or k=="attach" then return false end
                if k=="derivedOrientation" then return true, self.cam.derivedOrientation * self.cam.orientation.inverse end
                return true, self.cam[k]
        end, function (k,v)
                self.cam[k] = v
                return true
        end)

end

MonoCamera.attach = attach
MonoCamera.detach = detach

function MonoCamera:destroy()
        self.viewport:destroy()
        self.cam:destroy()
end







AnaglyphCamera = { }





function AnaglyphCamera.new(scnmgr,parent,render_target,z,left,top,width,height)
        local self = { }

        if parent == nil then error("parent was nil") end

        self.scnmgr = scnmgr
        self.root = parent:createChild()
        self.name = "Anaglyph Camera"
        self.type = "Anaglyph Camera"


        self.leftCam = scnmgr:createCamera("LeftEyeCam")
        self.leftCam.orientation = Quat(90,V_EAST)
        self.root:attachObject(self.leftCam)
        
        self.leftRenderTarget = render_target
        self.leftViewport = render_target:addViewport(self.leftCam,z,left,top,width,height)

        self.rightCam = scnmgr:createCamera("RightEyeCam")
        self.rightCam.orientation = self.leftCam.orientation
        self.root:attachObject(self.rightCam)

        local w,h = self.leftViewport.actualWidth, self.leftViewport.actualHeight

        self.rightTex = Texture("RightEye","2D",w,h,1,0,0x200)
        self.rightViewport = self.rightTex.renderTexture:addViewport(self.rightCam,1,0,0,1,1)
        self.rightViewport.overlaysEnabled = false

        self.leftViewport:addCompositor("system/AnaglyphCompositor")
        self.leftViewport:setCompositorEnabled("system/AnaglyphCompositor",true)

        make_instance(self,AnaglyphCamera,function (k)
                if k=="FOV" or k=="autoAspectRatio" or k=="nearClipDistance"
                            or k=="farClipDistance" or k=="useRenderingDistance"
                            or k=="polygonMode" then
                        return true, self.leftCam[k]
                elseif k=="separation" then
                        return true, self.rightCam.position.x - self.leftCam.position.x
                elseif k=="frustumOffset" then
                        return true, self.leftCam.frustumOffsetX - self.rightCam.frustumOffsetX
                elseif k=="focalLength" then
                        return true, self.leftCam.focalLength
                end
                if k=="derivedOrientation" then return true, self.root.derivedOrientation end
                if k=="derivedPosition" then return true, self.root.derivedPosition end
                if k=="parentSceneNode" then return true, self.root.parentSceneNode end
        end, function (k,v)
                if k=="FOV" or k=="autoAspectRatio" or k=="nearClipDistance"
                            or k=="farClipDistance" or k=="useRenderingDistance"
                            or k=="polygonMode" then
                        self.leftCam[k] = v
                        self.rightCam[k] = v
                        return true
                elseif k=="focalLength" then
                        self.leftCam.focalLength = v
                        self.rightCam.focalLength = v
                        return true
                elseif k=="separation" then
                        self.leftCam.position = Vector3(-v/2,0,0)
                        self.rightCam.position = Vector3(v/2,0,0)
                        return true
                elseif k=="frustumOffset" then
                        self.leftCam.frustumOffsetX =  v/2
                        self.rightCam.frustumOffsetX = -v/2
                        return true
                end
        end, function ()
                return "AnaglyphCamera: focalLength="..self.focalLength..
                                      " frustumOffset="..self.frustumOffset..
                                      " separation="..self.separation..
                                      " FOV="..self.FOV..
                                      " nearClipDistance="..self.nearClipDistance..
                                      " farClipDistance="..self.farClipDistance
        end)

        self:setup()

        return self
end

function AnaglyphCamera:destroy()
        self.leftViewport:destroy()
        self.leftCam:destroy()
        self.rightViewport:destroy()
        self.rightCam:destroy()
        self.root:destroy()
        remove_texture("RightEye")
end

AnaglyphCamera.attach = attach
AnaglyphCamera.detach = detach

function AnaglyphCamera:setup(min,max,distance,monitor_height,eye_separation)
        distance = distance or 0.6
        monitor_height = monitor_height or 0.27
        eye_separation = eye_separation or 0.06

        min = min or distance -0.05
        max = max or min + 0.1

        local s = 2*math.tan(math.rad(self.FOV/2))/monitor_height * (eye_separation * (1-distance/max))
        self.frustumOffset = s
        self.focalLength = 1

        local c = 2*math.tan(math.rad(self.FOV/2))/monitor_height * (eye_separation * (1-distance/min))
        c = self.nearClipDistance * (s - c)
        self.separation = c
end

function AnaglyphCamera:resizeTexture()
        self.leftViewport:removeCompositor("AnaglyphCompositor")
        local w,h = self.leftViewport.actualWidth, self.leftViewport.actualHeight
        remove_texture("RightEye")
        self.rightTex = Texture("RightEye","TEX_TYPE_2D",w,h,1,0,0x200)
        self.rightViewport = self.rightTex.renderTexture:addViewport(self.rightCam,1,0,0,1,1)
        self.rightViewport.overlaysEnabled = false

        self.leftViewport:addCompositor("AnaglyphCompositor")
        self.leftViewport:setCompositorEnabled("AnaglyphCompositor",true)
end



CrossCamera = { }

function CrossCamera.new(scnmgr,parent,render_target,z,left,top,width,height)
        local self = { }

        self.scnmgr = scnmgr
        self.root = parent:createChild()
        self.name = "Cross Camera"
        self.type = "Cross Camera"

        self.leftCam = scnmgr:createCamera("LeftEyeCam")
        self.leftCam.orientation = Quat(90,V_EAST)
        self.root:attachObject(self.leftCam)

        self.leftViewport = render_target:addViewport(self.leftCam,z,left+0.5,top,width/2,height)

        self.rightCam = scnmgr:createCamera("RightEyeCam")
        self.rightCam.orientation = self.leftCam.orientation
        self.root:attachObject(self.rightCam)

        self.rightViewport = render_target:addViewport(self.rightCam,z+1,left,top,width/2,height)
        self.rightViewport.overlaysEnabled = true
        
        make_instance(self,CrossCamera,function (k)
                if k=="FOV" or k=="autoAspectRatio" or k=="nearClipDistance"
                            or k=="farClipDistance" or k=="useRenderingDistance"                           
                            or k=="polygonMode" then
                        return true, self.leftCam[k]
                elseif k=="separation" then
                        return true, self.rightCam.position.x - self.leftCam.position.x                
                elseif k=="frustumOffset" then
                        return true, self.leftCam.frustumOffsetX - self.rightCam.frustumOffsetX        
                elseif k=="focalLength" then
                        return true, self.leftCam.focalLength
                end     
                if k=="derivedOrientation" then return true, self.cam.derivedOrientation * self.cam.orientation.inverse end
        end, function (k,v)
                if k=="FOV" or k=="autoAspectRatio" or k=="nearClipDistance"
                            or k=="farClipDistance" or k=="useRenderingDistance"
                            or k=="polygonMode" then
                        self.leftCam[k] = v
                        self.rightCam[k] = v
                        return true
                elseif k=="focalLength" then
                        self.leftCam.focalLength = v
                        self.rightCam.focalLength = v
                        return true
                elseif k=="separation" then
                        self.leftCam.position = Vector3(-v/2,0,0)
                        self.rightCam.position = Vector3(v/2,0,0)
                        return true
                elseif k=="frustumOffset" then
                        self.leftCam.frustumOffsetX =  v/2
                        self.rightCam.frustumOffsetX = -v/2
                        return true
                end
        end, function ()
                return "CrossCamera: focalLength="..self.focalLength..
                                   " frustumOffset="..self.frustumOffset..
                                   " separation="..self.separation..
                                   " FOV="..self.FOV..
                                   " nearClipDistance="..self.nearClipDistance..
                                   " farClipDistance="..self.farClipDistance
        end)

        self:setup()

        return self

end

function CrossCamera:destroy()
        self.leftViewport:destroy()
        self.leftCam:destroy()
        self.rightViewport:destroy()
        self.rightCam:destroy()
        self.root:destroy()
end

CrossCamera.attach = attach
CrossCamera.detach = detach

CrossCamera.setup = AnaglyphCamera.setup


-- vim: shiftwidth=8:tabstop=8:expandtab
