-- (c) David Cunningham 2009, Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php

print("Loading user configuration")


safe_include("/user_cfg.lua")

user_cfg = user_cfg or { }
debug_cfg = debug_cfg or { }
user_core_bindings = user_core_bindings or { }
user_ghost_bindings = user_ghost_bindings or { }
user_drive_bindings = user_drive_bindings or { }


local user_cfg_default = {
    shadow = true;
    shadowBackwards = true;
    shadowRes = 1024;
    shadowUseDiffuse = true;
    shadowPCFBilinear = true;
    shadowPCFTaps = 2;
    shadowPCFSpread = 4;
    shadowPCFEdgeDither = true;
    shadowPCSSPadding = 0.7;
    shadowPCSSSplits = 3;
    shadowPCSSStart = 0.2;
    shadowPCSSEnd0 = 20;
    shadowPCSSEnd1 = 50;
    shadowPCSSEnd2 = 200;
    shadowPCSSAdj0 = 3;
    shadowPCSSAdj1 = 1;
    shadowPCSSAdj2 = 1;
    shadowPCSSSpreadFactor0 = 1;
    shadowPCSSSpreadFactor1 = 1;
    shadowPCSSSpreadFactor2 = 0.28;
    shadowFadeStart = 150;
    fullscreen = false;
    res = {800,600};
    visibility = 1;
    graphicsRAM = 128;
    anisotropy = 4;
    lockMemory = true;
    textureShrink = 0;
}

local user_cfg_spec = {
    shadow = { "one of", false, true };
    shadowBackwards = { "one of", false, true };
    shadowRes = { "one of", 128,256,512,1024,2048,4096 };
    shadowUseDiffuse = { "one of", false, true };
    shadowPCFBilinear = { "one of", false, true };
    shadowPCFTaps = { "int range", 1, 6 };
    shadowPCFSpread = { "range", 0, 40 };
    shadowPCFEdgeDither = { "one of", false, true };
    shadowPCSSPadding = { "range", 0, 100 };
    shadowPCSSSplits = { "int range", 1, 4 };
    shadowPCSSStart = { "range", 0, 10000 };
    shadowPCSSEnd0 = { "range", 0, 10000 };
    shadowPCSSEnd1 = { "range", 0, 10000 };
    shadowPCSSEnd2 = { "range", 0, 10000 };
    shadowPCSSAdj0 = { "range", 0, 10000 };
    shadowPCSSAdj1 = { "range", 0, 10000 };
    shadowPCSSAdj2 = { "range", 0, 10000 };
    shadowPCSSSpreadFactor0 = { "range", 0, 20 };
    shadowPCSSSpreadFactor1 = { "range", 0, 20 };
    shadowPCSSSpreadFactor2 = { "range", 0, 20 };
    shadowFadeStart = { "range", 0, 10000 };
    res = { "table", 2, {"int range", 1, 4096}, {"int range", 1, 4096} };
    fullscreen = { "one of", false, true };
    visibility = { "range", 0, 5 }; 
    graphicsRAM = { "int range", 0, 2048 };
    lockMemory = { "one of", false, true };
    anisotropy = { "int range", 0, 16 };
    textureShrink = { "int range", 0, 4 };
}           
            

local debug_cfg_default = {
    shadowCast = true;
    shadowReceive = true;
    noVertexProcessing = false;
    noFragmentProcessing = false;
    normalMaps = true;
    diffuseMaps = true;
    specularMaps = true;
    translucencyMaps = true;
    vertexColours = true;
    filtering = true;
    fog = true;
    fixedFunction = false;
    boundingBoxes = false;
    debugShadow = false;
    debugShadowSize = 128;
    FOV = 55;
    farClip = 800;
    polygonMode = "SOLID";
    physicsWireFrame = false;
    physicsDebugWorld = true;
    textureAnimation = true;
    ambientOnly = false;
}

local debug_cfg_spec = {
    shadowCast = { "one of", false, true };
    shadowReceive = { "one of", false, true };
    noVertexProcessing = { "one of", false, true };
    noFragmentProcessing = { "one of", false, true };
    polygonMode = { "one of", "SOLID", "WIREFRAME", "POINTS" };
    normalMaps = { "one of", false, true };
    diffuseMaps = { "one of", false, true };
    specularMaps = { "one of", false, true };
    translucencyMaps = { "one of", false, true };
    vertexColours = { "one of", false, true };
    filtering = { "one of", false, true };
    fog = { "one of", false, true };
    fixedFunction = { "one of", false, true };
    boundingBoxes = { "one of", false, true };
    debugShadow = { "one of", false, 0, 1, 2 };
    debugShadowSize = { "int range", 1, 1024 };
    FOV = { "range", 0, 120 };
    farClip = { "range", 1, 10000 };
    physicsWireFrame = { "one of", false, true };
    physicsDebugWorld = { "one of", false, true };
    textureAnimation = { "one of", false, true };
    ambientOnly = { "one of", false, true };
}   



local default_user_core_bindings = {
    console = "Tab";
    record = "C+F12";
    screenShot = "F12";
    physicsPause = "F9";
    physicsSplitImpulse = "C+F10";
    physicsOneToOne = "F10";
    physicsMovieFrame = "C+F11";
    physicsStep = "F11";
    wireFrame = "F8";
    boundingBoxes = "C+F8";
    physicsWireFrame = "F7";
    physicsDebugWorld = "C+F7";
    skyPause = "F5";
    skyEdit = "C+F5";
    clearPlaced = "F3";
    clearProjectiles = "F4";
    toggleFullScreen = "A+Return";
    toggleChaseCamTrack = "F2";
}

local default_user_ghost_bindings = {
    forwards = "w";
    backwards = "s";
    strafeLeft = "a";
    strafeRight = "d";
    board = "f";
    ascend = "Space";
    descend = "Shift";
    fast = "right";
    teleportUp = "Return";
    teleportDown = "BackSpace";
}

local default_user_drive_bindings = {
    forwards = "w";
    backwards = "s";
    steerLeft = "a";
    steerRight = "d";
    specialLeft = "q";
    specialRight = "e";
    specialUp = "PageUp";
    specialDown = "PageDown";
    altUp = "Up";
    altDown = "Down";
    altLeft = "Left";
    altRight = "Right";
    abandon = "f";
    handbrake = "Space";
    zoomIn = {"up","S+v"};
    zoomOut = {"down","v"};
    boost = "left";
    realign = "Return";
    special = "BackSpace";
}


local core_binding_functions = {
    console = function() console.visible = not console.visible end;
    record = function() capturer:toggle() end;
    screenShot = function() capturer:singleScreenShot() end;
    physicsPause = function ()
        physics.enabled = not physics.enabled
        echo("Physics enabled: "..tostring(physics.enabled))
    end;
    physicsSplitImpulse = function ()
        pw.solverSplitImpulse = not pw.solverSplitImpulse
        echo("Physics splitImpulse: "..tostring(pw.solverSplitImpulse))
    end;
    physicsOneToOne = function ()
        physics.oneToOne = not physics.oneToOne
        echo("Physics one-to-one: "..tostring(physics.oneToOne))
    end;
    physicsMovieFrame = function () pw:pump(1/25) capturer:singleScreenShot() end;
    physicsStep = function () pw:pump(pw.stepSize) end;
    boundingBoxes = function() debug_cfg.boundingBoxes = not debug_cfg.boundingBoxes end;
    wireFrame = function()
        local pm = debug_cfg.polygonMode
        if pm == "SOLID" then
            debug_cfg.polygonMode = "WIREFRAME"
        elseif pm == "WIREFRAME" then 
            debug_cfg.polygonMode = "POINTS"
        else    
            debug_cfg.polygonMode = "SOLID"
        end     
    end;
    physicsWireFrame = function()
        set_physics_debug("DrawWireframe",not get_physics_debug("DrawWireframe"))
    end;
    physicsDebugWorld = function()
        physics.debugWorld = not physics.debugWorld
    end;
    skyPause = function() env.clockTicking = not env.clockTicking end;
    skyEdit = function() env:toggleEditMode() end;
    clearPlaced = clear_placed;
    clearProjectiles = clear_projectiles;
    toggleFullScreen = function ()
        user_cfg.fullscreen = not user_cfg.fullscreen
        -- avoid these keys getting 'stuck down' as we lose focus momentarily
        keyb_flush("Alt")
        keyb_flush("Return")
        ui:flush("Alt")
        ui:flush("Return")
    end;
    toggleChaseCamTrack = function () player_ctrl.toggleChaseCamTrack() end;
}

local ghost_binding_functions = {
    forwards = {function() player_ctrl.forwards=1 end, function() player_ctrl.forwards = 0 end};
    backwards = {function() player_ctrl.backwards=1 end, function() player_ctrl.backwards = 0 end};
    strafeLeft = {function() player_ctrl.left=1 end, function() player_ctrl.left = 0 end};
    strafeRight = {function() player_ctrl.right=1 end, function() player_ctrl.right = 0 end};
    board = function() player_ctrl:pickDrive() end;
    ascend = {function() player_ctrl.up=1 end, function() player_ctrl.up = 0 end};
    descend = {function() player_ctrl.down=1 end, function() player_ctrl.down = 0 end};
    fast = {function() player_ctrl.fast = true end, function() player_ctrl.fast = false end};
    teleportUp = {function() gfx.camPos.position = gfx.camPos.position+V_UP end, nil, true};
    teleportDown = {function() gfx.camPos.position = gfx.camPos.position+V_DOWN end, nil, true};
}

local drive_binding_functions = {
    forwards = {function() player_ctrl.vehicle:setPush(true) end, function() player_ctrl.vehicle:setPush(false) end};
    backwards = {function() player_ctrl.vehicle:setPull(true) end, function() player_ctrl.vehicle:setPull(false) end};
    steerLeft = {function() player_ctrl.vehicle:setShouldSteerLeft(true) end, function() player_ctrl.vehicle:setShouldSteerLeft(false) end};
    steerRight = {function() player_ctrl.vehicle:setShouldSteerRight(true) end, function() player_ctrl.vehicle:setShouldSteerRight(false) end};
    abandon = function() player_ctrl:abandonVehicle() end;
    handbrake = {function() player_ctrl.vehicle:setHandbrake(true) end, function() player_ctrl.vehicle:setHandbrake(false) end};
    zoomIn = {function() player_ctrl:zoomIn() end, nil, true};
    zoomOut = {function() player_ctrl:zoomOut() end, nil, true};
    boost = function() player_ctrl.vehicle:boost() end;
    realign = {function() player_ctrl.vehicle:realign() end, nil, true};
    specialUp = {function() player_ctrl.vehicle:setSpecialUp(true) end, function() player_ctrl.vehicle:setSpecialUp(false) end};
    specialDown = {function() player_ctrl.vehicle:setSpecialDown(true) end, function() player_ctrl.vehicle:setSpecialDown(false) end};
    specialLeft = {function() player_ctrl.vehicle:setSpecialLeft(true) end, function() player_ctrl.vehicle:setSpecialLeft(false) end};
    specialRight = {function() player_ctrl.vehicle:setSpecialRight(true) end, function() player_ctrl.vehicle:setSpecialRight(false) end};
    altUp = {function() player_ctrl.vehicle:setAltUp(true) end, function() player_ctrl.vehicle:setAltUp(false) end};
    altDown = {function() player_ctrl.vehicle:setAltDown(true) end, function() player_ctrl.vehicle:setAltDown(false) end};
    altLeft = {function() player_ctrl.vehicle:setAltLeft(true) end, function() player_ctrl.vehicle:setAltLeft(false) end};
    altRight = {function() player_ctrl.vehicle:setAltRight(true) end, function() player_ctrl.vehicle:setAltRight(false) end};
    special = function() player_ctrl.vehicle:special() end;
}


local function process_user_table(name, given, default)
    for k,v in pairs(given) do
        if default[k] == nil then
            echo(name.." contained unrecognised field \""..k.."\", ignoring.")
            given[k] = nil
        end
    end
    for k,v in pairs(default) do
        if given[k] == nil then
            given[k] = default[k]
        end
    end
end

process_user_table("user_cfg", user_cfg, user_cfg_default)
process_user_table("debug_cfg", debug_cfg, debug_cfg_default)
process_user_table("user_core_bindings", user_core_bindings, default_user_core_bindings)
process_user_table("user_ghost_bindings", user_ghost_bindings, default_user_ghost_bindings)
process_user_table("user_drive_bindings", user_drive_bindings, default_user_drive_bindings)


local function bind(name, data, functions, tab)
    local function bind_it(key)
        local data = functions[name]
        if type(data) == "table" then
            -- table[1,2] may be nil so unpack won't work
            tab.bind(tab, key, data[1], data[2], data[3])
        else
            tab.bind(tab, key, data)
        end
    end
    if type(data) == "table" then
        for _,key in ipairs(data) do
            bind_it(key)
        end
    else
        bind_it(data)
    end
end

local function process_bindings(bindings, functions, tab)
    for name,key in pairs(bindings) do
        bind(name, key, functions, tab)
    end
end

process_bindings(user_core_bindings, core_binding_functions, ui.coreBinds)
process_bindings(user_ghost_bindings, ghost_binding_functions, player_ctrl.ghostBinds)
process_bindings(user_drive_bindings, drive_binding_functions, player_ctrl.driveBinds)






local function commit(c, p, flush, partial)

    flush = flush or false
    partial = partial or false

    local reset_shadowmaps = false or flush
    local reset_pcss = false or flush
    local reset_shadows = false or flush
    local reset_debug_shadow = false or flush
    local reset_shaders = false or flush
    local reset_materials = false or flush
    local reset_fullscreen = false or flush

    for k,v in pairs(p) do
        if c[k] ~= v then
            c[k] = v
    
            if k == "shadow" then
                reset_shadows = true
                reset_shaders = true
                reset_materials = true
            elseif k == "shadowRes" then
                reset_shadowmaps = true
                reset_shaders = true
            elseif k == "shadowBackwards" then
                sm.shadowCasterRenderBackFaces = v
            elseif k == "shadowCast" then
                reset_shadows = true
            elseif k == "shadowReceive" then
                reset_shaders = true
                reset_materials = true
            elseif k == "shadowUseDiffuse" then
                reset_materials = true
            elseif k == "shadowPCFBilinear" then
                reset_shaders = true
            elseif k == "shadowPCFTaps" then
                reset_shaders = true
            elseif k == "shadowPCFSpread" then
                reset_shaders = true
            elseif k == "shadowPCFEdgeDither" then
                reset_shaders = true
            elseif k == "shadowPCSSPadding" then
                reset_shaders = true
                reset_pcss = true
            elseif k == "shadowPCSSSplits" then
                reset_pcss = true
                reset_shaders = true
                reset_materials = true
                reset_shadowmaps = true
            elseif k == "shadowPCSSStart" then
                reset_pcss = true
            elseif k == "shadowPCSSEnd0" then
                reset_shaders = true
                reset_pcss = true
            elseif k == "shadowPCSSEnd1" then
                reset_shaders = true
                reset_pcss = true
            elseif k == "shadowPCSSEnd2" then
                reset_shaders = true
                reset_pcss = true
            elseif k == "shadowPCSSEnd3" then
                reset_shaders = true
                reset_pcss = true
            elseif k == "shadowPCSSAdj0" then
                reset_pcss = true
            elseif k == "shadowPCSSAdj1" then
                reset_pcss = true
            elseif k == "shadowPCSSAdj2" then
                reset_pcss = true
            elseif k == "shadowPCSSAdj3" then
                reset_pcss = true
            elseif k == "shadowPCSSSpreadFactor0" then
                reset_shaders = true
            elseif k == "shadowPCSSSpreadFactor1" then
                reset_shaders = true
            elseif k == "shadowPCSSSpreadFactor2" then
                reset_shaders = true
            elseif k == "shadowPCSSSpreadFactor3" then
                reset_shaders = true
            elseif k == "shadowFadeStart" then
                reset_shaders = true

            elseif k == "textureShrink" then
                reset_materials = true
            elseif k == "FOV" then
                gfx.cam.FOV = v
            elseif k == "res" then
                reset_fullscreen = true
            elseif k == "fullscreen" then
                reset_fullscreen = true
            elseif k == "farClip" then
                gfx.cam.farClipDistance = v
                gfx.skyNode.scale = v * V_ID
            elseif k == "visibility" then
                streamer.visibility = v
            elseif k == "noVertexProcessing" then
                reset_materials = true
                reset_shaders = true
            elseif k == "noFragmentProcessing" then
                reset_materials = true
                reset_shaders = true
            elseif k == "graphicsRAM" then
                set_texture_budget(v*1024*1024)
                set_mesh_budget(0)
            elseif k == "lockMemory" then
                if v then mlockall() else munlockall() end

            elseif k == "polygonMode" then
                gfx.cam.polygonMode = v
            elseif k == "normalMaps" then
                reset_materials = true
            elseif k == "diffuseMaps" then
                reset_materials = true
            elseif k == "specularMaps" then
                reset_materials = true
            elseif k == "translucencyMaps" then
                reset_materials = true
            elseif k == "vertexColours" then
                reset_shaders = true
            elseif k == "anisotropy" then
                reset_materials = true
            elseif k == "filtering" then
                reset_materials = true
            elseif k == "textureAnimation" then
                reset_materials = true
            elseif k == "fog" then
                reset_shaders = true
            elseif k == "boundingBoxes" then
                sm.showBoundingBoxes = v
            elseif k == "physicsWireFrame" then
                set_physics_debug("DrawWireframe", v);
            elseif k == "physicsDebugWorld" then
                physics.debugWorld = v
            elseif k == "ambientOnly" then
                -- nothing to do if clock is ticking
            elseif k == "fixedFunction" then
                reset_materials = true
            elseif k == "debugShadow" then
                reset_debug_shadow = true
            elseif k == "debugShadowSize" then
                reset_debug_shadow = true
            else
                error("Unexpected: "..k)
            end
        end
    end

    if partial then return end

    if reset_shadowmaps then
        -- may have to turn shadows off, too
        sm:setShadowTextureCountPerLightType("DIRECTIONAL",user_cfg.shadowPCSSSplits)
        sm:setShadowTextureSettings(user_cfg.shadowRes, user_cfg.shadowPCSSSplits, "PF_FLOAT32_R");
    end

    if reset_pcss then
        local tab = {
                splits = user_cfg.shadowPCSSSplits,
                padding = user_cfg.shadowPCSSPadding,
                boundary0 = user_cfg.shadowPCSSStart
        }
        for i=0,tab.splits-1 do
            tab["boundary"..(i+1)] = user_cfg["shadowPCSSEnd"..i]
        end
        for i=0,tab.splits-1 do
            tab["optimalAdjustFactor"..i] = user_cfg["shadowPCSSAdj"..i]
        end
        sm:setShadowCameraSetup("PSSM",tab)
    end

    if reset_debug_shadow then
        if debug_cfg.debugShadow then
            gfx.debugTexturePane.material:setTextureName(0,0,0,sm:getShadowTexture(debug_cfg.debugShadow).name)
            gfx.debugTexturePane.visible = true
        else
            gfx.debugTexturePane.visible = false
        end
        local sz = debug_cfg.debugShadowSize
        gfx.debugTexturePane.resize = Hud.LEFT(-1,sz,sz)
    end

    if reset_shadows then
            sm.shadowTechnique = (user_cfg.shadow and debug_cfg.shadowCast) and "TEXTURE_MODULATIVE_INTEGRATED" or "NONE"
    end

    if reset_shaders then
        do_reset_shaders()
    end

    if reset_materials then
        do_reset_materials()
    end

    if reset_fullscreen then
        if user_cfg.fullscreen then
            if get_main_win().isFullScreen then
                if get_main_win().width == user_cfg.res[1] and get_main_win().height == user_cfg.res[2] then
                    -- nothing to do
                else
                    -- change res, keep fullscreen
                    -- drop to windowed mode to work around ogre bug
                    --get_main_win():setFullScreen(false, 640, 480)
                    get_main_win():setFullScreen(true, user_cfg.res[1], user_cfg.res[2])
                end
            else
                -- make fullscreen
                get_main_win():setFullScreen(true, user_cfg.res[1], user_cfg.res[2])
            end
        else
            if get_main_win().isFullScreen then
                -- make windowed
                get_main_win():setFullScreen(false, 640, 480)
            else
                -- changing res does you no good if you're windowed
            end
        end
    end
    
end

make_active_table(user_cfg, user_cfg_spec,  commit)
make_active_table(debug_cfg, debug_cfg_spec,  commit)

commit(user_cfg.c, user_cfg.p, false, true)
commit(debug_cfg.c, debug_cfg.p, false, true)
debug_cfg.autoUpdate = true
user_cfg.autoUpdate = true

commit(user_cfg.c, user_cfg.p, true, false)





function save_user_cfg(filename)
    filename = filename or "user_cfg.lua"
    local f = io.open(filename,"w")
    f:write([[
print('Reading user_cfg.lua')

-- This file is output automatically by Grit.
-- You may edit it, but stick to the basic format.
-- Any clever Lua code will be lost.
--
-- Note that if you are changing from a default value
-- to a custom value, don't forget to uncomment the line
-- (remove the leading -- ) otherwise it will not be
-- processed and your changes will be lost.

]])

    local function write_table(table_name, tab, defaults)
        f:write(table_name.." = {\n")
        local names, num, max_name_len = table.keys(tab,100)
        table.sort(names)
        for _,name in ipairs(names) do
            local key = tab[name]
            if key == defaults[name] then
                f:write('--')
            end
            f:write("    "..tostring(name)..(" "):rep(max_name_len-#name).." = "..dump(key,false)..";\n")
        end
        f:write("}\n\n")
    end

    -- use proposed rather than current settings, to avoid writing out the autoUpdate header
    write_table("user_cfg", user_cfg.p, user_cfg_default)
    write_table("debug_cfg", debug_cfg.p, debug_cfg_default)
    write_table("user_core_bindings", user_core_bindings, default_user_core_bindings)
    write_table("user_ghost_bindings", user_ghost_bindings, default_user_ghost_bindings)
    write_table("user_drive_bindings", user_drive_bindings, default_user_drive_bindings)

    f:close()
end


