-- (c) David Cunningham 2009, Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php

print ("Initialising script...")
io.stdout:setvbuf("no") -- no output buffering
collectgarbage("setpause",110) -- begin a gc cycle after blah% increase in ram use
collectgarbage("setstepmul",150) -- collect at blah% the rate of new object creation

set_texture_verbose(false)
set_mesh_verbose(false)
get_main_win().autoDeactivate = false --don't disable rendering when focus lost
--set_keyb_verbose(true)

initialise_all_resource_groups()

include("strict.lua")

include("abbrev.lua")

include("util.lua")

include("hud.lua")



print("Starting game engine...")


main = {
        shouldQuit = false,
        frameCallbacks = CallbackReg.new()
}

function main:quit()
        self.shouldQuit = true
end

function main:run (...)

        -- execute cmdline arguments on console
        local arg = ""
        for i=2,select('#',...) do
                arg = arg.." "..select(i,...)
        end
        if #arg > 0 then
                console:exec(arg:sub(2))
        end

        local main_win = get_main_win()

        local last_focus = true

        local failName = {}

        -- rendering loop
        while not clicked_close() and not main.shouldQuit do

                
                if last_focus and not have_focus() then
                        keyb_flush() -- get rid of any sticky keys
                        last_focus = false
                        ui:updateGrabbed()
                end

                if not last_focus and have_focus() then
                        last_focus = true
                        ui:updateGrabbed()
                end


                xpcall(function ()

                        pump() -- handle OS GUI events (e.g. resizing, window close)
                        local t = Timer()

                        failName.name = nil
                        main.frameCallbacks:executeExtended(function (name,cb,path,...)
                                failName.name = name
                                --t:reset()
                                if cb == nil then
                                        --echo(RED.."Callback was nil: "..name)
                                        return true
                                end
                                path_stack_push_dir(path)
                                local result = cb(...)
                                path_stack_pop()
                                --local us = t.us
                                --if us>5000 and name~="GFX.frameCallback" then
                                --        print("callback \""..name.."\" took "..us/1000 .."ms ")
                                --end
                                return result
                        end)
                        failName.name = nil

                        if main_win.isActive == false then
                                sleep(3E7)
                        end

                end,error_handler)

                if failName.name then
                        path_stack_pop()
                        echo("Removed frameCallback: "..failName.name)
                        main.frameCallbacks:removeByName(failName.name)
                end

        end

        save_user_cfg()
        
end



sm = get_sm()
streamer.gfx = sm.root
streamer:setBounds(Vector3(-4000,-4000,-4000),Vector3(4000,4000,4000))
pw = streamer.physics
pw.stepSize = 1/100
pw.maxSteps = 20
pw.useTriangleEdgeInfo = false



include("ui.lua")

include("anaglyph_camera.lua")

include("gfx.lua")

include("materials.lua")

include("physics.lua")

include("console_prompt.lua")
include("console.lua")

include("player_ctrl.lua")

include("capturer.lua")

include("configuration.lua")

include("env.lua")


include("/common/init.lua")

include("/vehicles/init.lua")


safe_include("/user_script.lua")

-- vim: shiftwidth=8:tabstop=8:expandtab
