-- (c) David Cunningham 2009, Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php

print("Loading friction_models.lua")


function global_friction_model (vehicle, mu, off_road, restitution, speed, drive_force, traction_control, mat)

        -- first compute limiting friction
        -- this is the force required to keep the object stationary
        -- no matter how fast it is going
        local num_wheels = #vehicle.wheels
        local body = vehicle.body
        local world = vehicle.world
        local mass = body.mass / num_wheels
        local limiting_force = speed * mass / world.stepSize

        -- reduce the amount of play this force has in the eventual consensus
        limiting_force = limiting_force * traction_control

        -- now incorporate the drive force (at right angles)
        local required_force = math.sqrt(limiting_force * limiting_force + drive_force * drive_force)
        if required_force == 0 then
                required_force = 1 -- avoid NaN a few lines below
        end

        -- compute them maximum frictional force the surface can provide
        mu = mu * (off_road and PhysMat.offRoadTyreFriction[mat] or PhysMat.roadTyreFriction[mat])
        local theoretical_max = mu * restitution

        --if theoretical_max < required_force then
        --        echo ("slip: "..required_force.." > "..theoretical_max)
        --end

        -- now we have the actual applied friction
        local applied_force = math.min(theoretical_max,required_force)

        -- split it back into the 2 components
        local applied_drive = drive_force  *  applied_force / required_force
        local applied_friction =  limiting_force  *  applied_force / required_force
        
        return applied_friction, applied_drive, traction_control < 0.5
end

-- vim: shiftwidth=8:tabstop=8:expandtab
