Plane = extends (ColClass) {
	castShadows = true;
	renderingDistance = 200;

	maxThrust = {};
	currentThrust = {};
	targetThrust = {};
	stepThrust = 100;

	liftCurves = {};
	dragCurves = {};
	pressureCenters = {};
	actionVectors = {};
	glideRatios = {};
	controlTypes = {};
	aoaLimits = {};
	lastaoa = {};
	lastlift = {};
	lastdrag = {};

	rotorCurves = {};
	rotorGlide = {};
	rotorAction = {};
	rotorPressure = {};
	rotorControl = {};
	rotorRPM = {};
	rotorRad = {};

	--Shamelessly copied from vehicle base.
	steerMax = 25;
        steerRate = 1.3;
        unsteerRate = 5;
        steerFastSpeed = 100;
        steerRateFast = 0.3;
        unsteerRateFast = 5;

	wheelSpinTractionControlMax = 1;
        wheelSpinTractionControlRate = 4;
        wheelSpinTractionControlMin = 0.01;

	smokeSystemArg = { longevity=2, grow=2, startAlpha=1, period=20 };
	lightSystemArg = { longevity=0, grow=0, startAlpha=1, period=0 };

	brakePlot = { [-10] = 100; [0] = 100; [10] = 2500; [20] = 2500; [40] = 2500; };
}

function Plane:init (class)
    self.needsFrameCallbacks = true
    
end

function Plane.activate (persistent,self,class,gfxRoot,physics)
        ColClass.activate(persistent,self,class,gfxRoot,physics)

	self.push = 0
	self.parked = true

    -- 0 means retracted, 1 means down
	self.gearState = 0.999
	self.desiredGearState = 1
        local f = function (k,v) return self.entity:getBoneId(k), v end
        self.gearDoors = tabmap(class.gearDoors or {}, f)
        self.gearStruts = tabmap(class.gearStruts or {}, f) 
        self.gearStrutNames = tabmap(class.gearStruts or {}, function (k,v) return self.entity:getBoneId(k), k end)

	self.driftHackTimer = 0

	self.altHackTimer=0

	local mpw = 0
	if #class.wheelInfo > 0 then mpw = self.body.mass/#class.wheelInfo end -- mass per wheel (hacky)
        local len = class.suspensionLength

	self.steer, self.shouldSteerLeft, self.shouldSteerRight = 0, 0, 0 
        self.tractionControl = 1
        local n = self.gfx

	self.maxThrust={}
	self.currentThrust={}
	self.targetThrust={}
	self.stepThrust=class.stepThrust

	--Engine defs go here.
	for i,tp in ipairs(class.thrustPlots) do
		self.maxThrust[i]=Plot(tp)
		self.currentThrust[i] = 0.0
		self.targetThrust[i] = 0.0
	end

	self.liftCurves = {};
	self.dragCurves = {};
	self.pressureCenters = {};
	self.actionVectors = {};
	self.glideRatios = {};
	self.aoaLimits = {};
	self.controlTypes = {};
	self.lastaoa = {};
	self.lastlift = {};
	self.lastdrag = {};

	self.smokeTimer=0
	self.smokeSystem = sm:createParticleSystem(persistent.name..":ParticleSystem")
	self.gfx:attachObject(self.smokeSystem)
        self.smokeSystem.materialName = "common/particles/Smoke"

	self.lightSystem = sm:createParticleSystem(persistent.name..":LightParticleSystem")
	self.gfx:attachObject(self.lightSystem)
	self.lightSystem.materialName = "common/particles/Light"
        self.lightSystem:update(0)

	--Surface defs here.
	for i, info in ipairs(class.surfaceInfo) do
		self.liftCurves[i]=Plot(info.lift)
		self.dragCurves[i]=Plot(info.drag)
		self.pressureCenters[i]=info.cpr
		self.actionVectors[i]=info.act
		self.glideRatios[i]=info.glide
		self.controlTypes[i]=info.ctype
		self.aoaLimits[i]=info.limitAoA
		self.lastaoa[i] = 0.0
		self.lastlift[i] = 0.0
		self.lastdrag[i] = 0.0
	end

	self.rotorCurves={}
	self.rotorPressure={}
	self.rotorAction={}
	self.rotorRPM={}
	self.rotorRad={}
	self.rotorGlide={}
	self.rotorControl={}
	self.rotorCyclicX={}
	self.rotorCyclicY={}

	self.collective=0.5
	self.collectiveStep=0.025
	self.collectiveTarget=0

	self.cyclicx=0
	self.cyclicy=0
	self.cyclicxTarget=0
	self.cyclicyTarget=0
	self.cyclicStep=0.1

	for i, info in ipairs(class.rotorInfo) do
		self.rotorCurves[i]=Plot(info.lift)
		self.rotorPressure[i]=info.cpr
		self.rotorAction[i]=info.act
		self.rotorRPM[i]=info.rpm
		self.rotorRad[i]=info.diameter/3
		self.rotorGlide[i]=info.glide
		self.rotorCyclicX[i]=info.cyclicx
		self.rotorCyclicY[i]=info.cyclicy
		self.rotorControl[i]=info.ctype
	end

	self.lights = {}
	for i, info in ipairs(class.lightInfo) do
		self.lights[i]={}
		self.lights[i].pos=info.pos
		self.lights[i].rad=info.rad
		self.lights[i].color=info.color
	end

	self.pitch, self.pitchUp, self.pitchDown=0,0,0
	self.roll, self.rollLeft, self.rollRight=0,0,0

	--self.arm = class.cmArm

	self.brakeCurve = Plot (class.brakePlot)

	self.wheels = {}
	self.drivenWheels = {}
	self.handbrakeWheels = {}

	for i, info in ipairs(class.wheelInfo) do
                local wmass = (info.massShare or 1) * mpw
                self.wheels[i] = WheelAssembly.new(self, info.aimBone, info.steerBone, info.axleBone, info.wheelBone,
                                                   info.rad, info.slack,
                                                   wmass, info.mu, info.offRoad, info.maxRestitution,
                                                   info.dampingFactor)
                self.wheels[i].steerFactor = info.steer or 0
                self.wheels[i].drive = false

                if info.handbrake then table.insert(self.handbrakeWheels,self.wheels[i]) end
        end

	self.body.updateCallback = function (px,py,pz,rw,rx,ry,rz)
                local node = self.gfx
                -- can be nil
                if node ~= nil and alive(node) then
                        node:setPosition(px,py,pz)
                        node:setOrientation(rw,rx,ry,rz)
                end
                foreach (self.wheels, WheelAssembly.updateGFX)
                persistent:updateSphere(px,py,pz,self.renderingDistance)
        end

	self.body.stepCallback = function()

                local body = self.body
                local world = body.world
                local velocity = body.linearVelocity
                local speed = #velocity

		--Altitude hack.
		--self.altHackTimer=self.altHackTimer+1
		--if self.altHackTimer>50 then
		--	print("Altitude",body.worldPosition.z)
		--	self.altHackTimer=0
		--end

		--Adjust CM.
		--local cmtorque = Vector3(0, 0, 9.8*body.mass):cross(body.worldOrientation * self.arm)
		--body:torque(cmtorque)

        local avx,avy,avz = body:getAngularVelocity()
		body:torque(mulnv(-1.5, avx,avy,avz))
		body:torque(mulnv(-2.0*dot(avx,avy,avz, avx,avy,avz), avx,avy,avz))

		--gfx.camChaseYaw.orientation = body.worldOrientation * Quat(0,0,0,1)
		--gfx.camChase.inheritOrientation = true

		--Process thrust.
		local thrust
		for i,tp in ipairs(self.targetThrust) do
			if self.targetThrust[i] > self.currentThrust[i] then
				self.currentThrust[i] = self.currentThrust[i] + self.stepThrust
			end
			if self.targetThrust[i] < self.currentThrust[i] then
				self.currentThrust[i] = self.currentThrust[i] - self.stepThrust
			end

			thrust = body.worldOrientation * Vector3(0, self.currentThrust[i], 0)
			--print("Engine",i,thrust,body.worldPosition)
			body:force(thrust, body.worldPosition + body.worldOrientation*Vector3(0,5,0))
		end

		--if self.pitch > 0 then self.pitch = self.pitch - 0.002 end
		--if self.pitch < 0 then self.pitch = self.pitch + 0.002 end

		--if self.roll > 0 then self.pitch = self.roll - 0.005 end
		--if self.roll < 0 then self.pitch = self.roll + 0.005 end

		--local target
		--target = self.pitchDown-self.pitchUp
		--self.pitch=self.pitchDown-self.pitchUp
		--if target ~= 0 then
		--	if self.pitch < target then self.pitch = self.pitch + 0.004 end
		--	if self.pitch > target then self.pitch = self.pitch - 0.004 end
		--end
		--target = self.rollRight-self.rollLeft
		--if target ~= 0 then
		--	if self.roll < target then self.roll = self.roll + 0.005 end
		--	if self.roll > target then self.roll = self.roll - 0.005 end
		--end
		self.pitch=self.pitchDown-self.pitchUp
		self.roll=self.rollRight-self.rollLeft

		--Process lift.
		local lift
		local drag
		local speedx
		local speedy
		local aoa
		local speed2
		local pos=Vector3(0, 0, 0)
		local pvel
		local s
		local glide
		for s, glide in ipairs(self.glideRatios) do
			--pvel=Vector3(body:getLocalVelocity(self.pressureCenters[3],false))
			--pvel=body.linearVelocity
            local px,py,pz = body:rotateToWorld(self.pressureCenters[s]:xyz())
			pvel=Vector3(body:getLocalVelocity(px,py,pz, false))
			speedx = pvel:dot(body.worldOrientation * Vector3(0,-1,0))
			speedy = pvel:dot(body.worldOrientation * self.actionVectors[s])
			pvel = speedx * (body.worldOrientation * Vector3(0,-1,0)) + speedy * (body.worldOrientation * self.actionVectors[s])
			speed2 = speedx*speedx + speedy*speedy
			aoa = -pvel.normalised:dot(body.worldOrientation * self.actionVectors[s])

			if self.controlTypes[s]=="rudder" then aoa=aoa+self.steer/300.0 end
			if self.controlTypes[s]=="elevator" then aoa=aoa+self.pitch end
			if self.controlTypes[s]=="lwing" then aoa=aoa+self.roll end
			if self.controlTypes[s]=="rwing" then aoa=aoa-self.roll end
			self.lastaoa[s]=aoa

			if aoa > 0.8 then aoa=0.8 end
			if aoa < -0.8 then aoa=-0.8 end

			--print("Surface",s,aoa,speed2)

			lift = speed2 * self.liftCurves[s][aoa]
			if aoa < self.aoaLimits[s][1] or aoa > self.aoaLimits[s][2] then lift = 0 end
			drag = speed2 * self.dragCurves[s][math.abs(aoa)] / glide
			pos:xyz(body.worldOrientation * self.pressureCenters[s] + body.worldPosition)
			body:force(lift*(body.worldOrientation * self.actionVectors[s]), pos)
			body:force(-drag*pvel.normalised, pos)

			self.lastlift[s]=lift
			self.lastdrag[s]=drag

			--print("Surface",s,aoa)
			--print("Surface", s, lift*(body.worldOrientation * self.actionVectors[s]))
		end

		--Process Rotors.
		if self.collective<self.collectiveTarget then self.collective=self.collective+self.collectiveStep end
		if self.collective>self.collectiveTarget then self.collective=self.collective-self.collectiveStep end

		if self.cyclicx<self.cyclicxTarget then self.cyclicx=self.cyclicx+self.cyclicStep end
		if self.cyclicx>self.cyclicxTarget then self.cyclicx=self.cyclicx-self.cyclicStep end
		if self.cyclicy<self.cyclicyTarget then self.cyclicy=self.cyclicy+self.cyclicStep end
		if self.cyclicy>self.cyclicyTarget then self.cyclicy=self.cyclicy-self.cyclicStep end

		local rtorque
		local vx,vy,vz
		local wr,v2
		local act
		for s, glide in ipairs(self.rotorGlide) do
			rtorque=0
			lift=0
			wr = self.rotorRPM[s] * self.rotorRad[s] * 0.104719755
                        local psx,psy,psz = (body.worldOrientation * self.rotorPressure[s]):xyz()
			pvel=Vector3(body:getLocalVelocity(psx,psy,psz, false))
			act=body.worldOrientation*self.rotorAction[s]
			pvel=(act:dot(pvel))*act
			vx,vy,vz = pvel:xyz()
			if self.rotorControl[s] == "rotor" then
				v2 = vx*vx + vy*vy
				--Lift and position should be adjsuted for cyclic and wind.
				lift = self.rotorCurves[s][self.collective]
				--Needs fix for negative RPM
				rtorque = - (lift/glide)*self.rotorRad[s]
				pos = body.worldPosition + body.worldOrientation * self.rotorPressure[s]
				--position offset for velocity.
				--pos = pos + (wr*wr/(wr*wr+v2))*Vector3(vx,vy,0)
				--Cyclic Control.
				pos = pos + self.cyclicx*(body.worldOrientation*self.rotorCyclicX[s])
				pos = pos + self.cyclicy*(body.worldOrientation*self.rotorCyclicY[s])
				--lift offset for velocity
				--lift = lift * (wr*wr+v2)/(wr*wr) + (wr-vz)/wr

				--print("Rotor",self.cyclicx,self.cyclicy)
			end
			if self.rotorControl[s] == "torque" then
				aoa=.5-self.steer/(2*class.steerMax)
				if aoa<0 then aoa=0 end
				if aoa>1 then aoa=1 end
				lift = self.rotorCurves[s][aoa]
				rtorque=0
				pos = body.worldPosition + body.worldOrientation * self.rotorPressure[s]
			end
			--print("Rotor",s)

			if self.parked then
				lift=0
				rtorque=0
			end
				
			body:force(lift*(body.worldOrientation * self.rotorAction[s]), pos)
			body:torque(mulnv(rtorque, body:rotateToWorld(self.rotorAction[s]:xyz())))
		end

		--Mark with smoke
		local arg = class.smokeSystemArg
                --self.smokeTimer = self.smokeTimer + 1
                local emit_smoke = false
                if self.smokeTimer >= arg.period then
                        self.smokeTimer = 0
                        emit_smoke = true
                end
		if emit_smoke then
			pos = (body.worldOrientation * Vector3(0,0,10)) + body.worldPosition
			--pos = body.worldPosition
                	px, py, pz = pos:xyz()
                	self.smokeSystem:addParticle(px,py,pz+0.25, math.random()*360, --pos,rot
				0.5+math.random(),0.5+math.random(), --scale
				1,1,1, --colour
				1*arg.startAlpha*(1-math.random()/3),
				0.2-0.4*math.random(), 0.2-0.4*math.random(), 0.6+math.random(),
				particle_smoke, arg, 0)
		end

                -- hack to stop parked vehicles drifting
                if self.parked or self.brake or self.handBrake then
                        -- if we're parked then soak up anything left over from last time
                        if self.driftHackTimer > 100 then
                                if speed<0.04 then
                                        body.linearVelocity = V_ZERO
                                        body.angularVelocity = V_ZERO
                                end
                        end
                end

                local forward_speed = velocity:dot(body.worldOrientation * V_FORWARDS)
                local mph = forward_speed * 60 * 60 / METRES_PER_MILE

                local steerTarget = self.shouldSteerRight + self.shouldSteerLeft
                local change = steerTarget - self.steer
                if math.abs(change) > 0.001 then
                        local rate
                        local fastness = clamp(mph/class.steerFastSpeed, 0, 1)
                        if between(0,steerTarget,self.steer) and math.abs(self.steer) > 0.001 then
                                rate = (1-fastness)*class.unsteerRate + (fastness)*class.unsteerRateFast
                        else
                                rate = (1-fastness)*class.steerRate + (fastness)*class.steerRateFast
                        end
                        change = clamp(change,-rate, rate)
                        self.steer = self.steer + change
                        for k,w in ipairs(self.wheels) do
                                w:setSteer(w.steerFactor * self.steer)
                        end
                end
                
                for _, wheel in ipairs(self.wheels) do
                        wheel.locked = self.parked
                        wheel:setTorque(0)
                end

                local tcstep = class.wheelSpinTractionControlRate * world.stepSize
                self.tractionControl = self.tractionControl + tcstep

                self.tractionControl = clamp(self.tractionControl,
                                             class.wheelSpinTractionControlMin,
                                             class.wheelSpinTractionControlMax)

                for _, wheel in ipairs(self.handbrakeWheels) do
                        if self.handbrake then
                                wheel.locked = true
                        end
                end

                local interval = world.stepSize
                for _, wheel in ipairs(self.wheels) do
                        if self.brake then
                                if self.handbrake then
                                        wheel.locked = true
                                else
                                        wheel:setTorque(-sign(mph)*self.brakeCurve[math.abs(mph)])
                                end
                        end
                        wheel:process(interval)
                        local rnd = math.random
                        local skidding = wheel.skidding
                        if not wheel.locked then
                                -- must be skidding due to wheelspin
                                -- so attenuate smoke based on wheelspin
                                skidding = skidding - invlerp(self.tractionControl,
                                                              class.wheelSpinTractionControlMin,
                                                              class.wheelSpinTractionControlMax)
                        end
                        
                end

        end

	self.body.stabiliseCallback = function ()

                local body = self.body
                local world = self.world

                -- hack to stop parked cars drifting
                if self.parked or self.brake or self.handBrake then
                        local v = body.linearVelocity
                        local gdir = world.gravity.normalised
                        -- derive the anti-gravity component of v
                        local vz = v:dot(gdir) * gdir
                        -- work out what our velocity will be after gravity is applied
                        local final_v = v + world.gravity*world.stepSize
                        -- if the resultant force of this frame was small (i.e. was jitter)
                        if #final_v<0.1 then
                                self.driftHackTimer = self.driftHackTimer + 1
                                if self.driftHackTimer > 100 then
                                        -- soak up this force (except for anti-gravity component)
                                        body.linearVelocity = vz
                                        body.angularVelocity = V_ZERO
                                end
                        end
                else
                        self.driftHackTimer = 0
                end
        end

        return self
end

function Plane:deactivate (instance)
        foreach(instance.wheels or {}, safe_destroy)
	safe_destroy(instance.smokeSystem)
	safe_destroy(instance.lightSystem)
        instance.wheels = {}
        instance.drivenWheels = {}
        instance.handbrakeWheels = {}
        return ColClass.deactivate(self, instance)
end

function Plane:frameCallback (instance, time)
    --local body = instance.body
    instance.smokeSystem:update(time)
    --Light System
    instance.lightSystem:update(time)
    for s,light in ipairs(instance.lights) do
        --pvel=Vector3(body:getLocalVelocity(body.worldOrientation * light.pos,false))
        --pos = (body.worldOrientation * light.pos) + body.worldPosition + world.stepSize*pvel
        --local pos = (body.worldOrientation * light.pos) + body.worldPosition
        local pos = (instance.gfx.derivedOrientation * light.pos) + instance.gfx.derivedPosition
        local px, py, pz = pos:xyz()
        local cx,cy,cz = light.color:xyz()
        instance.lightSystem:addParticle(px,py,pz, 0, --pos,rot
            light.rad,light.rad, --scale
            cx,cy,cz, --colour
            1.0, -- Alpha
            0, 0, 0, --Vx,Vy,Vz
            particle_smoke, self.lightSystemArg, 0)
    end
    local diff = instance.desiredGearState - instance.gearState
    if diff ~= 0 then
        local body = instance.body
        local class = self.class
        local period = self.gearCycle or 4
        diff = clamp(diff, -time/period, time/period)
        instance.gearState = instance.gearState + diff
        local e = instance.entity
        local t = instance.gearState
        for b,a in pairs(instance.gearStruts) do
            --e:setBoneOrientation(b,(Quat(e:getBoneInitialOrientation(b))*Quat(a[t],V_RIGHT)):wxyz())
            e:setBoneOrientationOffsetAngle(b,a[t],1,0,0)

            local tab = class.gearStrutsCol[instance.gearStrutNames[b]]
            local el = tab[1]
            local axisx,axisy,axisz = tab[2]:xyz()
            local qw,qx,qy,qz = quat_angle(a[t], axisx,axisy,axisz)
            local px,py,pz = tab[3]:xyz()
            px,py,pz = addvv(px,py,pz, mulqv(qw,qx,qy,qz, -px,-py,-pz))
            body:setPartOrientationOffset(el, qw,qx,qy,qz)
            body:setPartPositionOffset(el, px,py,pz)
        end
        for b,a in pairs(instance.gearDoors) do
            e:setBoneOrientationOffsetAngle(b,a[t],0,1,0)
        end
    end
    for k,w in ipairs(instance.wheels) do
        w:updatePos()
    end
end


function Plane:getSpeed ()
        if not self.activated then error("Not activated: "..self.persistent.name) end
        local rb = self.instance.body
        return #rb.linearVelocity
end

function Plane:setSpeed (speed)
        if not self.activated then error("Not activated: "..self.persistent.name) end
        local rb = self.instance.body
        rb.linearVelocity = rb.linearVelocity.normalised * speed
        rb:activate();
end

function Plane:boost (speed)
        speed = speed or 1
        self:setSpeed(self:getSpeed()+speed)
end

function Plane:setHandbrake (v)
        if not self.activated then error("Not activated: "..self.persistent.name) end
        self.instance.handbrake = v
end

function Plane:abandon ()
        if not self.activated then return end
        self = self.instance
        local body = self.body
        self.parked = body.linearVelocity:dot(body.worldOrientation * V_FORWARDS) < 0.5
        self.brake = false
        self.push = 0
        self.handbrake = false
end

function Plane:setPull(v)
	if not self.activated then error("Not activated: "..self.persistent.name) end
	if v then self.instance.collectiveTarget=0
	else self.instance.collectiveTarget=0.5 end
end

function Plane:setPush(v)
	if not self.activated then error("Not activated: "..self.persistent.name) end

	--print("Pushing", self.instance.body.worldPosition)

	self.instance.parked = false

	for i,_ in ipairs(self.instance.targetThrust) do
		--print("Pushing engine", i)
		if v then self.instance.targetThrust[i]=self.instance.maxThrust[i][0]
		else self.instance.targetThrust[i]=0 end
		--self.instance.targetThrust[i]=5000;
		--self.instance.currentThrust[i]=5000;
	end

	if v then self.instance.collectiveTarget=1.0
	else self.instance.collectiveTarget=0.5 end
end

function Plane:setShouldSteerLeft(v)
	if not self.activated then error("Not activated: "..self.persistent.name) end
        self.instance.shouldSteerLeft = v and -self.steerMax or 0
end

function Plane:setShouldSteerRight (v)
        if not self.activated then error("Not activated: "..self.persistent.name) end
        self.instance.shouldSteerRight = v and self.steerMax or 0
end

function Plane:setAltLeft(v)
	if not self.activated then error("Not activated: "..self.persistent.name) end
        self.instance.rollLeft=v and 0.13 or 0

	if v then self.instance.cyclicxTarget=1.0
	else self.instance.cyclicxTarget=0 end
end

function Plane:setAltRight (v)
        if not self.activated then error("Not activated: "..self.persistent.name) end
        self.instance.rollRight=v and 0.13 or 0

	if v then self.instance.cyclicxTarget=-1.0
	else self.instance.cyclicxTarget=0 end
end

function Plane:setAltDown(v)
	if not self.activated then error("Not activated: "..self.persistent.name) end
        self.instance.pitchUp=v and 0.13 or 0

	if v then self.instance.cyclicyTarget=1.0
	else self.instance.cyclicyTarget=0 end
end

function Plane:setAltUp (v)
        if not self.activated then error("Not activated: "..self.persistent.name) end
        self.instance.pitchDown=v and 0.18 or 0

	if v then self.instance.cyclicyTarget=-1.0
	else self.instance.cyclicyTarget=0 end
end

function Plane:special ()
        if not self.activated then error("Not activated: "..self.persistent.name) end
        self.instance.desiredGearState = 1-self.instance.desiredGearState
end

function Plane:warp (pos,orientation)
        pos = pos or player_ctrl:pickPos()+Vector3(0,0,2)
        orientation = orientation or gfx.cam.derivedOrientation
        self:activate()
        if not self.activated then error("Tried to activate but still not activated: "..self.persistent.name) end
        local rb = self.instance.body
        rb.worldPosition = pos
        rb.worldOrientation = orientation
        rb.linearVelocity = V_ZERO
        rb.angularVelocity = V_ZERO
end

function Plane:realign ()
        ColClass.realign(self)
        self.instance.broken = false
end

function Plane:setMu (mu)
        if not self.activated then error("Not activated: "..self.persistent.name) end
        for _,w in ipairs(self.instance.wheels) do
                w.mu = mu
        end
end

function Plane:reset ()
        if not self.activated then error("Not activated: "..self.persistent.name) end
        self.pos = self.instance.body.worldPosition
        self.rot = self.instance.body.worldOrientation
        self:deactivate()
        self:activate()
        player_ctrl:drive(self)
end

-- vim: ts=8:sw=8:et
